from sqlalchemy import create_engine
from sqlalchemy.orm import scoped_session, sessionmaker

def init_sqlalchemy(app, connect):
    """ Init SqlAlchemy

        :param app: Flask application
        :type app: Flask
        :param connect: Database connection string
        :type connect: str
        :rtype: (sqlalchemy.engine.Engine, sqlalchemy.orm.session.Session)
        """
    # Connect
    engine = create_engine(connect, convert_unicode=True)
    db_session = scoped_session(sessionmaker(autocommit=False, autoflush=False, bind=engine))

    # Models
    from overc.lib.db.models import Base
    Base.query = db_session.query_property()
    Base.metadata.create_all(bind=engine)  # TODO: remove automatic table creation

    # teardown
    @app.teardown_appcontext
    def shutdown_session(exception=None):
        db_session.remove()

    # Finish
    return (engine, db_session)



#! /usr/bin/env python

import os
import logging
import multiprocessing

from overc import OvercApplication
from overc.lib.supervise import supervise_loop

# Load config
app_config_file = os.environ.get('OVERC_CONFIG', 'server.ini')
app_config = OvercApplication.loadConfigFile(app_config_file)

# Configure logging
logging.basicConfig(level=getattr(logging, app_config['LOGLEVEL']))

# Application
app = OvercApplication(
    __name__,
    app_config['INSTANCE_PATH'],
    app_config
)

# Supervisor thread
p = multiprocessing.Process(name='supervisor', target=supervise_loop, args=(app,))
p.daemon = True
p.start()


# Debug mode
if __name__ == '__main__':
    # Serve static files in debug mode
    from werkzeug.wsgi import SharedDataMiddleware
    from werkzeug.serving import run_simple

    app = SharedDataMiddleware(app.app, {
        '/static/ui': ('overc.src.bps.ui', 'static')
    }, cache=False)

    run_simple(application=app,
               hostname='0.0.0.0',
               port=5000,
               use_reloader=True,
               use_debugger=True
    )
